"""add Asset.soc_udi_event_id

Revision ID: 3e43d3274d16
Revises: 31f251554682
Create Date: 2018-09-21 15:07:00.816216

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "3e43d3274d16"
down_revision = "31f251554682"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("asset", sa.Column("soc_udi_event_id", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("asset", "soc_udi_event_id")
    # ### end Alembic commands ###
