"""add_timezone_column_in_bvp_users_table

Revision ID: a328412b4623
Revises: 8abe32ffa204
Create Date: 2018-04-24 15:31:18.190845

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "a328412b4623"
down_revision = "8abe32ffa204"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "bvp_users", sa.Column("timezone", sa.String(length=255), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("bvp_users", "timezone")
    # ### end Alembic commands ###
