"""time-series-indexes

Revision ID: a5b970eadb3b
Revises: bddc5e9f72a3
Create Date: 2020-11-05 17:15:31.767627

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = "a5b970eadb3b"
down_revision = "bddc5e9f72a3"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(op.f("power_asset_id_idx"), "power", ["asset_id"], unique=False)
    op.create_index(op.f("power_datetime_idx"), "power", ["datetime"], unique=False)
    op.create_index(op.f("price_datetime_idx"), "price", ["datetime"], unique=False)
    op.create_index(op.f("price_market_id_idx"), "price", ["market_id"], unique=False)
    op.create_index(op.f("weather_datetime_idx"), "weather", ["datetime"], unique=False)
    op.create_index(
        op.f("weather_sensor_id_idx"), "weather", ["sensor_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("weather_sensor_id_idx"), table_name="weather")
    op.drop_index(op.f("weather_datetime_idx"), table_name="weather")
    op.drop_index(op.f("price_market_id_idx"), table_name="price")
    op.drop_index(op.f("price_datetime_idx"), table_name="price")
    op.drop_index(op.f("power_datetime_idx"), table_name="power")
    op.drop_index(op.f("power_asset_id_idx"), table_name="power")
    # ### end Alembic commands ###
