"""create_latest_task_run_table

Revision ID: b087ce8b529f
Revises: 91a938bfa5a8
Create Date: 2018-06-14 12:06:05.037547

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "b087ce8b529f"
down_revision = "91a938bfa5a8"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "latest_task_run",
        sa.Column("name", sa.String(length=80), nullable=False),
        sa.Column("datetime", sa.DateTime(timezone=True), nullable=True),
        sa.Column("status", sa.Boolean(), nullable=True),
        sa.PrimaryKeyConstraint("name"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("latest_task_run")
    # ### end Alembic commands ###
